@@ -3,12 +3,16 @@ package Moose::Meta::Method;
 use strict;
 use warnings;
 
-our $VERSION   = '0.82';
+our $VERSION   = '1.23';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
+use Class::MOP::MiniTrait;
+
 use base 'Class::MOP::Method';
 
+Class::MOP::MiniTrait::apply(__PACKAGE__, 'Moose::Meta::Object::Trait');
+
 sub _error_thrower {
     my $self = shift;
     ( ref $self && $self->associated_metaclass ) || "Moose::Meta::Class";
@@ -41,10 +45,10 @@ Moose::Meta::Method - A Moose Method metaclass
 
 =head1 DESCRIPTION
 
-This class is a subclass of L<Class::MOP::Class> that provides
+This class is a subclass of L<Class::MOP::Method> that provides
 additional Moose-specific functionality, all of which is private.
 
-To understand this class, you should read the the L<Class::MOP::Class>
+To understand this class, you should read the the L<Class::MOP::Method>
 documentation.
 
 =head1 INHERITANCE
@@ -53,9 +57,7 @@ C<Moose::Meta::Method> is a subclass of L<Class::MOP::Method>.
 
 =head1 BUGS
 
-All complex software has bugs lurking in it, and this module is no
-exception. If you find a bug please either email me, or add the bug
-to cpan-RT.
+See L<Moose/BUGS> for details on reporting bugs.
 
 =head1 AUTHOR
 
@@ -63,7 +65,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2009 by Infinity Interactive, Inc.
+Copyright 2006-2010 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
