@@ -6,7 +6,7 @@ use metaclass;
 
 use Scalar::Util 'blessed';
 
-our $VERSION   = '1.09';
+our $VERSION   = '1.23';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -57,7 +57,6 @@ sub new {
             roles        => [ @composition_roles ],
             cache        => 1,
         );
-        $meta->add_method(meta => sub { $meta });
         $class = $meta->name;
     }
 
@@ -101,6 +100,11 @@ sub get_method_list {
     return keys %{ $self->_method_map };
 }
 
+sub _get_local_methods {
+    my $self = shift;
+    return values %{ $self->_method_map };
+}
+
 sub has_method {
     my ($self, $method_name) = @_;
 
